@@ -1,14 +1,14 @@
-/** strptime workaround (for oa macos leopard) 
+/** strptime workaround (for oa macos leopard)
   * This strptime follows the man strptime (2001-11-12)
   *		conforming to SUSv2, POSIX.1-2001
   *
-  * This very simple version of strptime has no: 
+  * This very simple version of strptime has no:
   * - E alternatives
   * - O alternatives
   * - Glibc additions
   * - Does not process week numbers
   * - Does not properly processes year day
-  * 
+  *
   * LICENSE
   * Copyright (c) 2008, NLnet Labs, Matthijs Mekking
   * All rights reserved.
@@ -55,16 +55,16 @@
 static const char *abb_weekdays[] = {
 	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", NULL
 };
-static const char *full_weekdays[] = { 
-	"Sunday", "Monday", "Tuesday", "Wednesday", 
+static const char *full_weekdays[] = {
+	"Sunday", "Monday", "Tuesday", "Wednesday",
 	"Thursday", "Friday", "Saturday", NULL
 };
 static const char *abb_months[] = {
-	"Jan", "Feb", "Mar", "Apr", "May", "Jun", 
+	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec", NULL
 };
 static const char *full_months[] = {
-	"January", "February", "March", "April", "May", "June", 
+	"January", "February", "March", "April", "May", "June",
 	"July", "August", "September", "October", "November", "December", NULL
 };
 static const char *ampm[] = {
@@ -89,35 +89,34 @@ match_string(const char **buf, const char **strs)
 static int
 str2int(const char **buf, int max)
 {
-	int ret=0, count=0; 
+	int ret=0, count=0;
 
 	while (*buf[0] != '\0' && isdigit(*buf[0]) && count<max) {
 		ret = ret*10 + (*buf[0] - '0');
 		(*buf)++;
-		count++; 
+		count++;
 	}
 
 	if (!count)
-		return -1; 
-	return ret; 
+		return -1;
+	return ret;
 }
 
 /** Converts the character string s to values which are stored in tm
-  * using the format specified by format 
+  * using the format specified by format
  **/
 char *
 nsd_strptime(const char *s, const char *format, struct tm *tm)
 {
-	int c, alt_format, ret; 
-	int split_year = 0; 
-	char *tmp;
+	int c, alt_format, ret;
+	int split_year = 0;
 
 	while ((c = *format) != '\0') {
-		alt_format = 0; 
+		alt_format = 0;
 
 		/* whitespace, literal or format */
 		if (isspace(c)) { /* whitespace */
-			/** whitespace matches zero or more whitespace characters in the 
+			/** whitespace matches zero or more whitespace characters in the
 			  * input string.
 			 **/
 			while (isspace(*s))
@@ -125,7 +124,7 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 		}
 		else if (c == '%') { /* format */
 			format++;
-			c = *format; 
+			c = *format;
 			switch (c) {
 				case '%': /* %% is converted to % */
 					if (*s != c) {
@@ -135,20 +134,20 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 					break;
 				case 'a': /* weekday name, abbreviated or full */
 				case 'A':
-					ret = match_string(&s, full_weekdays); 
-					if (ret < 0) 
-						ret = match_string(&s, abb_weekdays); 
+					ret = match_string(&s, full_weekdays);
+					if (ret < 0)
+						ret = match_string(&s, abb_weekdays);
 					if (ret < 0) {
 						return NULL;
 					}
-					tm->tm_wday = ret; 
+					tm->tm_wday = ret;
 					break;
 				case 'b': /* month name, abbreviated or full */
 				case 'B':
 				case 'h':
-					ret = match_string(&s, full_months); 
-					if (ret < 0) 
-						ret = match_string(&s, abb_months); 
+					ret = match_string(&s, full_months);
+					if (ret < 0)
+						ret = match_string(&s, abb_months);
 					if (ret < 0) {
 						return NULL;
 					}
@@ -170,16 +169,16 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 					}
 					else {
 						tm->tm_year = ret*100 - TM_YEAR_BASE;
-						split_year = 1; 
+						split_year = 1;
 					}
 					break;
 				case 'd': /* day of month */
 				case 'e':
 					ret = str2int(&s, 2);
-					if (ret < 1 || ret > 31) { /* must be in [01,31] */ 
+					if (ret < 1 || ret > 31) { /* must be in [01,31] */
 						return NULL;
 					}
-					tm->tm_mday = ret; 
+					tm->tm_mday = ret;
 					break;
 				case 'D': /* equivalent to %m/%d/%y */
 					if (!(s = nsd_strptime(s, "%m/%d/%y", tm))) {
@@ -191,7 +190,7 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 					if (ret < 0 || ret > 23) { /* must be in [00,23] */
 						return NULL;
 					}
-					tm->tm_hour = ret; 
+					tm->tm_hour = ret;
 					break;
 				case 'I': /* 12hr clock hour */
 					ret = str2int(&s, 2);
@@ -200,28 +199,29 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 					}
 					if (ret == 12) /* actually [0,11] */
 						ret = 0;
-					tm->tm_hour = ret; 
+					tm->tm_hour = ret;
 					break;
 				case 'j': /* day of year */
 					ret = str2int(&s, 2);
 					if (ret < 1 || ret > 366) { /* must be in [001,366] */
-						return NULL; 
+						return NULL;
 					}
-					tm->tm_yday = ret; 
+					tm->tm_yday = ret;
 					break;
 				case 'm': /* month */
 					ret = str2int(&s, 2);
 					if (ret < 1 || ret > 12) { /* must be in [01,12] */
-						return NULL; 
+						return NULL;
 					}
-					tm->tm_mon = ret; 
+					/* months go from 0-11 */
+					tm->tm_mon = (ret-1);
 					break;
 				case 'M': /* minute */
 					ret = str2int(&s, 2);
 					if (ret < 0 || ret > 59) { /* must be in [00,59] */
-						return NULL; 
+						return NULL;
 					}
-					tm->tm_min = ret; 
+					tm->tm_min = ret;
 					break;
 				case 'n': /* arbitrary whitespace */
 				case 't':
@@ -229,7 +229,7 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 						s++;
 					break;
 				case 'p': /* am pm */
-					ret = match_string(&s, ampm); 
+					ret = match_string(&s, ampm);
 					if (ret < 0) {
 						return NULL;
 					}
@@ -255,9 +255,9 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 					/* 60 may occur for leap seconds */
 					/* earlier 61 was also allowed */
 					if (ret < 0 || ret > 60) { /* must be in [00,60] */
-						return NULL; 
+						return NULL;
 					}
-					tm->tm_sec = ret; 
+					tm->tm_sec = ret;
 					break;
 				case 'T': /* equivalent of %H:%M:%S */
 					if (!(s = nsd_strptime(s, "%H:%M:%S", tm))) {
@@ -267,25 +267,25 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 				case 'U': /* week number, with the first Sun of Jan being w1 */
 					ret = str2int(&s, 2);
 					if (ret < 0 || ret > 53) { /* must be in [00,53] */
-						return NULL; 
+						return NULL;
 					}
-					/** it is hard (and not necessary for nsd) to determine time 
+					/** it is hard (and not necessary for nsd) to determine time
 					  * data from week number.
 					 **/
 					break;
 				case 'w': /* day of week */
 					ret = str2int(&s, 1);
-					if (ret < 0 || ret > 6) { /* must be in [0,6] */ 
+					if (ret < 0 || ret > 6) { /* must be in [0,6] */
 						return NULL;
 					}
-					tm->tm_wday = ret; 
+					tm->tm_wday = ret;
 					break;
 				case 'W': /* week number, with the first Mon of Jan being w1 */
 					ret = str2int(&s, 2);
 					if (ret < 0 || ret > 53) { /* must be in [00,53] */
-						return NULL; 
+						return NULL;
 					}
-					/** it is hard (and not necessary for nsd) to determine time 
+					/** it is hard (and not necessary for nsd) to determine time
 					  * data from week number.
 					 **/
 					break;
@@ -302,7 +302,7 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 				case 'y': /* last two digits of a year */
 					ret = str2int(&s, 2);
 					if (ret < 0 || ret > 99) { /* must be in [00,99] */
-						return NULL; 
+						return NULL;
 					}
 					if (split_year) {
 						tm->tm_year = ((tm->tm_year/100) * 100) + ret;
@@ -310,14 +310,14 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 					else {
 						split_year = 1;
 
-						/** currently: 
-						  * if in [0,68] we are in 21th century, 
-						  * if in [69,99] we are in 20th century. 
+						/** currently:
+						  * if in [0,68] we are in 21th century,
+						  * if in [69,99] we are in 20th century.
 						 **/
 						if (ret < 69) /* 2000 */
 							ret += 100;
 						tm->tm_year = ret;
-					}	
+					}
 					break;
 				case 'Y': /* year */
 					ret = str2int(&s, 4);
@@ -334,15 +334,15 @@ nsd_strptime(const char *s, const char *format, struct tm *tm)
 		}
 		else { /* literal */
 			/* if input cannot match format, return NULL */
-			if (*s != c) 
+			if (*s != c)
 				return NULL;
 			s++;
 		}
 
 		format++;
-	}	
-	
-	/* return pointer to remainder of s */	
+	}
+
+	/* return pointer to remainder of s */
 	return (char*) s;
 }
 
