@@ -140,6 +140,9 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_MAILA	254	/* A request for mail agent RRs (Obsolete - see MX) */
 #define TYPE_ANY	255	/* any type (wildcard) */
 
+#define TYPE_DLV	32769	/* RFC 4431 */
+#define PSEUDO_TYPE_DLV	RRTYPE_DESCRIPTORS_LENGTH
+
 #define MAXLABELLEN	63
 #define MAXDOMAINLEN	255
 
