@@ -46,18 +46,24 @@ struct domain
 	domain_type *wildcard_child_closest_match;
 	rrset_type  *rrsets;
 #ifdef NSEC3
-	/* (if nsec3 chain complete) always the covering nsec3 record */ 
+	/* (if nsec3 chain complete) always the covering nsec3 record */
 	domain_type *nsec3_cover;
 	/* the nsec3 that covers the wildcard child of this domain. */
 	domain_type *nsec3_wcard_child_cover;
 	/* for the DS case we must answer on the parent side of zone cut */
 	domain_type *nsec3_ds_parent_cover;
 	/* the NSEC3 domain that has a hash-base32 <= than this dname. */
-	/* or NULL (no smaller on within this zone) */
+	/* or NULL (no smaller one within this zone)
+	 * this variable is used to look up the NSEC3 record that matches
+	 * or covers a given b64-encoded-hash-string domain name.
+	 * The result of the lookup is stored in the *_cover variables.
+	 * The variable makes it possible to perform a rbtree lookup for
+	 * a name, then take this 'jump' to the previous element that contains
+	 * an NSEC3 record, with hopefully the correct parameters. */
 	domain_type *nsec3_lookup;
 #endif
 	uint32_t     number; /* Unique domain name number.  */
-	
+
 	/*
 	 * This domain name exists (see wildcard clarification draft).
 	 */
@@ -119,10 +125,10 @@ struct rrset
  */
 union rdata_atom
 {
-	/* RDATA_WF_COMPRESSED_DNAME, RDATA_WF_UNCOMPRESSED_DNAME.  */
+	/* RDATA_WF_COMPRESSED_DNAME, RDATA_WF_UNCOMPRESSED_DNAME */
 	domain_type *domain;
 
-	/* Default.  */
+	/* Default. */
 	uint16_t    *data;
 };
 
@@ -170,10 +176,10 @@ domain_type *domain_table_insert(domain_table_type *table,
 /*
  * Iterate over all the domain names in the domain tree.
  */
-typedef void (*domain_table_iterator_type)(domain_type *node,
+typedef int (*domain_table_iterator_type)(domain_type *node,
 					   void *user_data);
 
-void domain_table_iterate(domain_table_type *table,
+int domain_table_iterate(domain_table_type *table,
 			  domain_table_iterator_type iterator,
 			  void *user_data);
 
@@ -233,6 +239,8 @@ struct namedb
 	size_t	  	  zone_count;
 	char              *filename;
 	FILE              *fd;
+	/* the timestamp on the ixfr.db file */
+	struct timeval	  diff_timestamp;
 	/* the CRC on the nsd.db file and position of CRC in the db file */
 	uint32_t	  crc;
 	off_t		  crc_pos;
@@ -281,6 +289,7 @@ int namedb_lookup (struct namedb    *db,
 /* pass number of children (to alloc in dirty array */
 struct namedb *namedb_open(const char *filename, struct nsd_options* opt,
 	size_t num_children);
+void namedb_fd_close(struct namedb *db);
 void namedb_close(struct namedb *db);
 
 static inline int
