@@ -12,6 +12,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <limits.h>
+#include "tsig.h"
 #include "options.h"
 #include "util.h"
 #include "dname.h"
@@ -49,7 +50,7 @@ extern int optind;
 
 #define SERV_GET_INT(NAME, VAR) 		\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 	\
-		printf("%d\n", opt->NAME); 	\
+		printf("%d\n", (int) opt->NAME); 	\
 		return; 			\
 	}
 
@@ -90,7 +91,7 @@ usage(void)
 	fprintf(stderr, "usage: nsd-checkconf [-v|-h] [-o option] [-z zonename]\n");
 	fprintf(stderr, "                     [-s keyname] <configfilename>\n");
 	fprintf(stderr, "       Checks NSD configuration file for errors.\n");
-	fprintf(stderr, "       Version %s. Report bugs to <%s>.\n\n", 
+	fprintf(stderr, "       Version %s. Report bugs to <%s>.\n\n",
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 	fprintf(stderr, "Use with a configfile as argument to check syntax.\n");
 	fprintf(stderr, "Use with -o, -z or -s options to query the configuration.\n\n");
@@ -102,12 +103,12 @@ usage(void)
 	exit(1);
 }
 
-static void 
+static void
 print_string_var(const char* varname, const char* value)
 {
 	if (!value) {
 		printf("\t#%s\n", varname);
-	} else { 
+	} else {
 		printf("\t%s \"%s\"\n", varname, value);
 	}
 }
@@ -121,7 +122,7 @@ quote(const char *v)
 		printf("%s\n", v);
 }
 
-static void 
+static void
 quote_acl(acl_options_t* acl)
 {
 	while(acl)
@@ -133,7 +134,7 @@ quote_acl(acl_options_t* acl)
 	}
 }
 
-static void 
+static void
 print_acl(const char* varname, acl_options_t* acl)
 {
 	while(acl)
@@ -141,6 +142,8 @@ print_acl(const char* varname, acl_options_t* acl)
 		printf("\t%s ", varname);
 		if(acl->use_axfr_only)
 			printf("AXFR ");
+		if(acl->allow_udp)
+			printf("UDP ");
 		printf("%s %s\n", acl->ip_address_spec,
 			acl->nokey?"NOKEY":(acl->blocked?"BLOCKED":
 			(acl->key_name?acl->key_name:"(null)")));
@@ -220,6 +223,9 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 				ZONE_GET_ACL(provide_xfr, o);
 				ZONE_GET_ACL(allow_notify, o);
 				ZONE_GET_ACL(notify, o);
+				ZONE_GET_BIN(notify_retry, o);
+				ZONE_GET_ACL(outgoing_interface, o);
+				ZONE_GET_BIN(allow_axfr_fallback, o);
 				printf("Zone option not handled: %s %s\n", z, o);
 				exit(1);
 			}
@@ -233,9 +239,11 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 		SERV_GET_BIN(debug_mode, o);
 		SERV_GET_BIN(ip4_only, o);
 		SERV_GET_BIN(ip6_only, o);
+		SERV_GET_BIN(hide_version, o);
 		/* str */
 		SERV_GET_STR(database, o);
 		SERV_GET_STR(identity, o);
+		SERV_GET_STR(nsid, o);
 		SERV_GET_STR(logfile, o);
 		SERV_GET_STR(pidfile, o);
 		SERV_GET_STR(chroot, o);
@@ -247,8 +255,13 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 		/* int */
 		SERV_GET_INT(server_count, o);
 		SERV_GET_INT(tcp_count, o);
+		SERV_GET_INT(tcp_query_count, o);
+		SERV_GET_INT(tcp_timeout, o);
+		SERV_GET_INT(ipv4_edns_size, o);
+		SERV_GET_INT(ipv6_edns_size, o);
 		SERV_GET_INT(statistics, o);
 		SERV_GET_INT(xfrd_reload_timeout, o);
+		SERV_GET_INT(verbosity, o);
 
 		if(strcasecmp(o, "zones") == 0) {
 			RBTREE_FOR(zone, zone_options_t*, opt->zone_options)
@@ -260,7 +273,7 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 	}
 }
 
-void 
+void
 config_test_print_server(nsd_options_t* opt)
 {
 	ip_address_option_t* ip;
@@ -272,11 +285,17 @@ config_test_print_server(nsd_options_t* opt)
 	printf("\tdebug-mode: %s\n", opt->debug_mode?"yes":"no");
 	printf("\tip4-only: %s\n", opt->ip4_only?"yes":"no");
 	printf("\tip6-only: %s\n", opt->ip6_only?"yes":"no");
+	printf("\thide-version: %s\n", opt->hide_version?"yes":"no");
 	print_string_var("database:", opt->database);
 	print_string_var("identity:", opt->identity);
+	print_string_var("nsid:", opt->nsid);
 	print_string_var("logfile:", opt->logfile);
 	printf("\tserver_count: %d\n", opt->server_count);
 	printf("\ttcp_count: %d\n", opt->tcp_count);
+	printf("\ttcp_query_count: %d\n", opt->tcp_query_count);
+	printf("\ttcp_timeout: %d\n", opt->tcp_timeout);
+	printf("\tipv4-edns-size: %d\n", (int) opt->ipv4_edns_size);
+	printf("\tipv6-edns-size: %d\n", (int) opt->ipv6_edns_size);
 	print_string_var("pidfile:", opt->pidfile);
 	print_string_var("port:", opt->port);
 	printf("\tstatistics: %d\n", opt->statistics);
@@ -306,13 +325,16 @@ config_test_print_server(nsd_options_t* opt)
 		print_string_var("zonefile:", zone->zonefile);
 		print_acl("allow-notify:", zone->allow_notify);
 		print_acl("request-xfr:", zone->request_xfr);
+		printf("\tnotify-retry: %d\n", zone->notify_retry);
 		print_acl("notify:", zone->notify);
 		print_acl("provide-xfr:", zone->provide_xfr);
+		print_acl("outgoing-interface:", zone->outgoing_interface);
+		printf("\tallow-axfr-fallback: %s\n", zone->allow_axfr_fallback?"yes":"no");
 	}
-	
+
 }
 
-static int 
+static int
 additional_checks(nsd_options_t* opt, const char* filename)
 {
 	ip_address_option_t* ip = opt->ip_addresses;
@@ -324,7 +346,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 		num++;
 		ip = ip->next;
 	}
-	if(num >= MAX_INTERFACES) {
+	if(num > MAX_INTERFACES) {
 		fprintf(stderr, "%s: too many interfaces (ip-address:) specified.\n", filename);
 		errors ++;
 	}
@@ -359,9 +381,10 @@ additional_checks(nsd_options_t* opt, const char* filename)
 			fprintf(stderr, "%s: cannot base64 decode tsig secret: for key %s.\n", filename, key->name);
 			errors ++;
 		}
-		if(strcmp(key->algorithm, "hmac-md5") != 0)
+		if(tsig_get_algorithm_by_name(key->algorithm) != NULL)
 		{
-			fprintf(stderr, "%s: bad tsig algorithm: for key %s.\n", filename, key->name);
+			fprintf(stderr, "%s: bad tsig algorithm %s: for key \
+%s.\n", filename, key->algorithm, key->name);
 			errors ++;
 		}
 	}
@@ -369,7 +392,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 #ifndef BIND8_STATS
 	if(opt->statistics > 0)
 	{
-		fprintf(stderr, "%s: 'statistics: %d' but BIND 8 statistics feature not enabled.\n", 
+		fprintf(stderr, "%s: 'statistics: %d' but BIND 8 statistics feature not enabled.\n",
 			filename, opt->statistics);
 		errors ++;
 	}
@@ -377,7 +400,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 #ifndef HAVE_CHROOT
 	if(opt->chroot != 0)
 	{
-		fprintf(stderr, "%s: chroot %s given. chroot not supported on this platform.\n", 
+		fprintf(stderr, "%s: chroot %s given. chroot not supported on this platform.\n",
 			filename, opt->chroot);
 		errors ++;
 	}
@@ -394,45 +417,45 @@ additional_checks(nsd_options_t* opt, const char* filename)
                 int l = strlen(opt->chroot);
 
                 if (strncmp(opt->chroot, opt->pidfile, l) != 0) {
-			fprintf(stderr, "%s: pidfile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: pidfile %s is not relative to chroot %s.\n",
 				filename, opt->pidfile, opt->chroot);
 			errors ++;
-                } 
+                }
 		if (strncmp(opt->chroot, opt->database, l) != 0) {
-			fprintf(stderr, "%s: databasefile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: databasefile %s is not relative to chroot %s.\n",
 				filename, opt->database, opt->chroot);
 			errors ++;
                 }
 		if (strncmp(opt->chroot, opt->difffile, l) != 0) {
-			fprintf(stderr, "%s: difffile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: difffile %s is not relative to chroot %s.\n",
 				filename, opt->difffile, opt->chroot);
 			errors ++;
                 }
 		if (strncmp(opt->chroot, opt->xfrdfile, l) != 0) {
-			fprintf(stderr, "%s: xfrdfile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: xfrdfile %s is not relative to chroot %s.\n",
 				filename, opt->xfrdfile, opt->chroot);
 			errors ++;
                 }
         }
 	if (atoi(opt->port) <= 0) {
-		fprintf(stderr, "%s: port number '%s' is not a positive number.\n", 
+		fprintf(stderr, "%s: port number '%s' is not a positive number.\n",
 			filename, opt->port);
 		errors ++;
 	}
 	if(errors != 0) {
 		fprintf(stderr, "%s: %d semantic errors in %d zones, %d keys.\n",
-			filename, errors, (int)nsd_options_num_zones(opt), 
+			filename, errors, (int)nsd_options_num_zones(opt),
 			(int)opt->numkeys);
 	}
-	
+
 	return (errors == 0);
 }
 
-int 
+int
 main(int argc, char* argv[])
 {
 	int c;
-	int verbose = 0; 
+	int verbose = 0;
 	const char * conf_opt = NULL; /* what option do you want? Can be NULL -> print all */
 	const char * conf_zone = NULL; /* what zone are we talking about */
 	const char * conf_key = NULL; /* what key is needed */
@@ -475,9 +498,9 @@ main(int argc, char* argv[])
 		config_print_zone(options, conf_key, underscore(conf_opt), conf_zone);
 	} else {
 		if (verbose) {
-			printf("# Read file %s: %d zones, %d keys.\n", 
-				configfile, 
-				(int)nsd_options_num_zones(options), 
+			printf("# Read file %s: %d zones, %d keys.\n",
+				configfile,
+				(int)nsd_options_num_zones(options),
 				(int)options->numkeys);
 			config_test_print_server(options);
 		}
