@@ -30,20 +30,26 @@ nsd_options_t* nsd_options_create(region_type* region)
 	nsd_options_t* opt;
 	opt = (nsd_options_t*)region_alloc(region, sizeof(nsd_options_t));
 	opt->region = region;
-	opt->zone_options = rbtree_create(region, 
+	opt->zone_options = rbtree_create(region,
 		(int (*)(const void *, const void *)) dname_compare);
 	opt->keys = NULL;
 	opt->numkeys = 0;
 	opt->ip_addresses = NULL;
 	opt->debug_mode = 0;
 	opt->verbosity = 0;
+	opt->hide_version = 0;
 	opt->ip4_only = 0;
 	opt->ip6_only = 0;
 	opt->database = DBFILE;
 	opt->identity = 0;
+	opt->nsid = 0;
 	opt->logfile = 0;
 	opt->server_count = 1;
 	opt->tcp_count = 10;
+	opt->tcp_query_count = 0;
+	opt->tcp_timeout = TCP_TIMEOUT;
+	opt->ipv4_edns_size = EDNS_MAX_MESSAGE_LEN;
+	opt->ipv6_edns_size = EDNS_MAX_MESSAGE_LEN;
 	opt->pidfile = PIDFILE;
 	opt->port = UDP_PORT;
 /* deprecated?	opt->port = TCP_PORT; */
@@ -62,7 +68,7 @@ int nsd_options_insert_zone(nsd_options_t* opt, zone_options_t* zone)
 {
 	/* create dname for lookup */
 	const dname_type* dname = dname_parse(opt->region, zone->name);
-	if(!dname) 
+	if(!dname)
 		return 0;
 	zone->node.key = dname;
 	if(!rbtree_insert(opt->zone_options, (rbnode_t*)zone))
@@ -76,7 +82,7 @@ int parse_options_file(nsd_options_t* opt, const char* file)
 	zone_options_t* zone;
 	acl_options_t* acl;
 
-	if(!cfg_parser) 
+	if(!cfg_parser)
 		cfg_parser = (config_parser_state_t*)region_alloc(
 			opt->region, sizeof(config_parser_state_t));
 	cfg_parser->filename = file;
@@ -101,77 +107,77 @@ int parse_options_file(nsd_options_t* opt, const char* file)
 		return 0;
 	}
 	c_in = in;
-        c_parse();
+	c_parse();
 	fclose(in);
 
 	if(cfg_parser->current_zone) {
-		if(!cfg_parser->current_zone->name) 
+		if(!cfg_parser->current_zone->name)
 			c_error("last zone has no name");
 		else {
-			if(!nsd_options_insert_zone(opt, 
+			if(!nsd_options_insert_zone(opt,
 				cfg_parser->current_zone))
 				c_error("duplicate zone");
 		}
-		if(!cfg_parser->current_zone->zonefile) 
+		if(!cfg_parser->current_zone->zonefile)
 			c_error("last zone has no zonefile");
 	}
 	if(opt->keys)
 	{
-		if(!opt->keys->name) 
+		if(!opt->keys->name)
 			c_error("last key has no name");
-		if(!opt->keys->algorithm) 
+		if(!opt->keys->algorithm)
 			c_error("last key has no algorithm");
-		if(!opt->keys->secret) 
+		if(!opt->keys->secret)
 			c_error("last key has no secret blob");
 	}
 	RBTREE_FOR(zone, zone_options_t*, opt->zone_options)
 	{
-		if(!zone->name) 
+		if(!zone->name)
 			continue;
-		if(!zone->zonefile) 
+		if(!zone->zonefile)
 			continue;
 		/* lookup keys for acls */
 		for(acl=zone->allow_notify; acl; acl=acl->next)
 		{
-			if(acl->nokey || acl->blocked) 
+			if(acl->nokey || acl->blocked)
 				continue;
 			acl->key_options = key_options_find(opt, acl->key_name);
-			if(!acl->key_options) 
+			if(!acl->key_options)
 				c_error_msg("key %s in zone %s could not be found",
 					acl->key_name, zone->name);
 		}
 		for(acl=zone->notify; acl; acl=acl->next)
 		{
-			if(acl->nokey || acl->blocked) 
+			if(acl->nokey || acl->blocked)
 				continue;
 			acl->key_options = key_options_find(opt, acl->key_name);
-			if(!acl->key_options) 
+			if(!acl->key_options)
 				c_error_msg("key %s in zone %s could not be found",
 					acl->key_name, zone->name);
 		}
 		for(acl=zone->request_xfr; acl; acl=acl->next)
 		{
-			if(acl->nokey || acl->blocked) 
+			if(acl->nokey || acl->blocked)
 				continue;
 			acl->key_options = key_options_find(opt, acl->key_name);
-			if(!acl->key_options) 
+			if(!acl->key_options)
 				c_error_msg("key %s in zone %s could not be found",
 					acl->key_name, zone->name);
 		}
 		for(acl=zone->provide_xfr; acl; acl=acl->next)
 		{
-			if(acl->nokey || acl->blocked) 
+			if(acl->nokey || acl->blocked)
 				continue;
 			acl->key_options = key_options_find(opt, acl->key_name);
-			if(!acl->key_options) 
+			if(!acl->key_options)
 				c_error_msg("key %s in zone %s could not be found",
 					acl->key_name, zone->name);
 		}
 	}
-	
+
 	if(cfg_parser->errors > 0)
 	{
-        	fprintf(stderr, "read %s failed: %d errors in configuration file\n", 
+        	fprintf(stderr, "read %s failed: %d errors in configuration file\n",
 			cfg_parser->filename,
 			cfg_parser->errors);
 		return 0;
@@ -218,7 +224,10 @@ zone_options_t* zone_options_create(region_type* region)
 	zone->allow_notify = 0;
 	zone->request_xfr = 0;
 	zone->notify = 0;
+	zone->notify_retry = 5;
 	zone->provide_xfr = 0;
+	zone->outgoing_interface = 0;
+	zone->allow_axfr_fallback = 1;
 	return zone;
 }
 
@@ -240,14 +249,14 @@ key_options_t* key_options_find(nsd_options_t* opt, const char* name)
 {
 	key_options_t* key = opt->keys;
 	while(key) {
-		if(strcmp(key->name, name)==0) 
+		if(strcmp(key->name, name)==0)
 			return key;
 		key = key->next;
 	}
 	return 0;
 }
 
-int acl_check_incoming(acl_options_t* acl, struct query* q, 
+int acl_check_incoming(acl_options_t* acl, struct query* q,
 	acl_options_t** reason)
 {
 	/* check each acl element.
@@ -294,9 +303,9 @@ int acl_addr_matches(acl_options_t* acl, struct query* q)
 #ifdef INET6
 		struct sockaddr_storage* addr_storage = (struct sockaddr_storage*)&q->addr;
 		struct sockaddr_in6* addr = (struct sockaddr_in6*)&q->addr;
-		if(addr_storage->ss_family != AF_INET6) 
+		if(addr_storage->ss_family != AF_INET6)
 			return 0;
-		if(acl->port != 0 && acl->port != ntohs(addr->sin6_port)) 
+		if(acl->port != 0 && acl->port != ntohs(addr->sin6_port))
 			return 0;
 		switch(acl->rangetype) {
 		case acl_range_mask:
@@ -312,7 +321,7 @@ int acl_addr_matches(acl_options_t* acl, struct query* q)
 			break;
 		case acl_range_single:
 		default:
-			if(memcmp(&addr->sin6_addr, &acl->addr.addr6, 
+			if(memcmp(&addr->sin6_addr, &acl->addr.addr6,
 				sizeof(struct in6_addr)) != 0)
 				return 0;
 			break;
@@ -325,9 +334,9 @@ int acl_addr_matches(acl_options_t* acl, struct query* q)
 	else
 	{
 		struct sockaddr_in* addr = (struct sockaddr_in*)&q->addr;
-		if(addr->sin_family != AF_INET) 
+		if(addr->sin_family != AF_INET)
 			return 0;
-		if(acl->port != 0 && acl->port != ntohs(addr->sin_port)) 
+		if(acl->port != 0 && acl->port != ntohs(addr->sin_port))
 			return 0;
 		switch(acl->rangetype) {
 		case acl_range_mask:
@@ -343,7 +352,7 @@ int acl_addr_matches(acl_options_t* acl, struct query* q)
 			break;
 		case acl_range_single:
 		default:
-			if(memcmp(&addr->sin_addr, &acl->addr.addr, 
+			if(memcmp(&addr->sin_addr, &acl->addr.addr,
 				sizeof(struct in_addr)) != 0)
 				return 0;
 			break;
@@ -383,17 +392,17 @@ int acl_addr_match_range(uint32_t* minval, uint32_t* x, uint32_t* maxval, size_t
 	{
 		/* if outside bounds, we are done */
 		if(checkmin)
-			if(minval[i] > x[i]) 
+			if(minval[i] > x[i])
 				return 0;
 		if(checkmax)
-			if(maxval[i] < x[i]) 
+			if(maxval[i] < x[i])
 				return 0;
 		/* if x is equal to a bound, that bound needs further checks */
 		if(checkmin && minval[i]!=x[i])
 			checkmin = 0;
 		if(checkmax && maxval[i]!=x[i])
 			checkmax = 0;
-		if(!checkmin && !checkmax) 
+		if(!checkmin && !checkmax)
 			return 1; /* will always match */
 	}
 	return 1;
@@ -401,7 +410,7 @@ int acl_addr_match_range(uint32_t* minval, uint32_t* x, uint32_t* maxval, size_t
 
 int acl_key_matches(acl_options_t* acl, struct query* q)
 {
-	if(acl->blocked) 
+	if(acl->blocked)
 		return 1;
 #ifdef TSIG
 	if(acl->nokey) {
@@ -422,25 +431,25 @@ int acl_key_matches(acl_options_t* acl, struct query* q)
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "keymatch fail no config"));
 		return 0; /* key not properly configged */
 	}
-	if(dname_compare(q->tsig.key_name, 
+	if(dname_compare(q->tsig.key_name,
 		acl->key_options->tsig_key->name) != 0) {
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "keymatch fail wrong key name"));
 		return 0; /* wrong key name */
 	}
-	if(strcmp(q->tsig.algorithm->short_name,
+	if(tsig_strlowercmp(q->tsig.algorithm->short_name,
 		acl->key_options->algorithm) != 0) {
 		DEBUG(DEBUG_XFRD,2, (LOG_ERR, "query tsig wrong algorithm"));
 		return 0; /* no such algo */
 	}
 	return 1;
 #else
-	if(acl->nokey) 
+	if(acl->nokey)
 		return 1;
 	return 0;
 #endif
 }
 
-int 
+int
 acl_same_host(acl_options_t* a, acl_options_t* b)
 {
 	if(a->is_ipv6 && !b->is_ipv6)
@@ -452,20 +461,20 @@ acl_same_host(acl_options_t* a, acl_options_t* b)
 	if(a->rangetype != b->rangetype)
 		return 0;
 	if(!a->is_ipv6) {
-		if(memcmp(&a->addr.addr, &b->addr.addr, 
+		if(memcmp(&a->addr.addr, &b->addr.addr,
 		   sizeof(struct in_addr)) != 0)
 			return 0;
 		if(a->rangetype != acl_range_single &&
-		   memcmp(&a->range_mask.addr, &b->range_mask.addr, 
+		   memcmp(&a->range_mask.addr, &b->range_mask.addr,
 		   sizeof(struct in_addr)) != 0)
 			return 0;
 	} else {
 #ifdef INET6
-		if(memcmp(&a->addr.addr6, &b->addr.addr6, 
+		if(memcmp(&a->addr.addr6, &b->addr.addr6,
 		   sizeof(struct in6_addr)) != 0)
 			return 0;
 		if(a->rangetype != acl_range_single &&
-		   memcmp(&a->range_mask.addr6, &b->range_mask.addr6, 
+		   memcmp(&a->range_mask.addr6, &b->range_mask.addr6,
 		   sizeof(struct in6_addr)) != 0)
 			return 0;
 #else
@@ -598,6 +607,8 @@ acl_options_t* parse_acl_info(region_type* region, char* ip, const char* key)
 	/* ip */
 	acl->ip_address_spec = region_strdup(region, ip);
 	acl->use_axfr_only = 0;
+	acl->allow_udp = 0;
+	acl->ixfr_disabled = 0;
 	acl->key_options = 0;
 	acl->is_ipv6 = 0;
 	acl->port = 0;
@@ -612,10 +623,10 @@ acl_options_t* parse_acl_info(region_type* region, char* ip, const char* key)
 	if(parse_acl_is_ipv6(ip)) {
 		acl->is_ipv6 = 1;
 #ifdef INET6
-		if(inet_pton(AF_INET6, ip, &acl->addr.addr6) != 1) 
+		if(inet_pton(AF_INET6, ip, &acl->addr.addr6) != 1)
 			c_error_msg("Bad ip6 address '%s'", ip);
-		if(acl->rangetype==acl_range_mask || acl->rangetype==acl_range_minmax) 
-			if(inet_pton(AF_INET6, p, &acl->range_mask.addr6) != 1) 
+		if(acl->rangetype==acl_range_mask || acl->rangetype==acl_range_minmax)
+			if(inet_pton(AF_INET6, p, &acl->range_mask.addr6) != 1)
 				c_error_msg("Bad ip6 address mask '%s'", p);
 		if(acl->rangetype==acl_range_subnet)
 			parse_acl_range_subnet(p, &acl->range_mask.addr6, 128);
@@ -624,10 +635,10 @@ acl_options_t* parse_acl_info(region_type* region, char* ip, const char* key)
 #endif /* INET6 */
 	} else {
 		acl->is_ipv6 = 0;
-		if(inet_pton(AF_INET, ip, &acl->addr.addr) != 1) 
+		if(inet_pton(AF_INET, ip, &acl->addr.addr) != 1)
 			c_error_msg("Bad ip4 address '%s'", ip);
-		if(acl->rangetype==acl_range_mask || acl->rangetype==acl_range_minmax) 
-			if(inet_pton(AF_INET, p, &acl->range_mask.addr) != 1) 
+		if(acl->rangetype==acl_range_mask || acl->rangetype==acl_range_minmax)
+			if(inet_pton(AF_INET, p, &acl->range_mask.addr) != 1)
 				c_error_msg("Bad ip4 address mask '%s'", p);
 		if(acl->rangetype==acl_range_subnet)
 			parse_acl_range_subnet(p, &acl->range_mask.addr, 32);
@@ -649,3 +660,8 @@ acl_options_t* parse_acl_info(region_type* region, char* ip, const char* key)
 	}
 	return acl;
 }
+
+void nsd_options_destroy(nsd_options_t* opt)
+{
+	region_destroy(opt->region);
+}
