@@ -34,12 +34,13 @@ struct nsd_options {
 	/* list of keys defined */
 	key_options_t* keys;
 	size_t numkeys;
-	
+
 	/* list of ip adresses to bind to (or NULL for all) */
 	ip_address_option_t* ip_addresses;
 
 	int debug_mode;
 	int verbosity;
+	int hide_version;
 	int ip4_only;
 	int ip6_only;
 	const char* database;
@@ -47,6 +48,10 @@ struct nsd_options {
 	const char* logfile;
 	int server_count;
 	int tcp_count;
+	int tcp_query_count;
+	int tcp_timeout;
+	size_t ipv4_edns_size;
+	size_t ipv6_edns_size;
 	const char* pidfile;
 	const char* port;
 	int statistics;
@@ -55,9 +60,10 @@ struct nsd_options {
 	const char* zonesdir;
 	const char* difffile;
 	const char* xfrdfile;
+	const char* nsid;
 	int xfrd_reload_timeout;
-	
-	region_type* region; 
+
+	region_type* region;
 };
 
 struct ipaddress_option {
@@ -79,6 +85,9 @@ struct zone_options {
 	acl_options_t* request_xfr;
 	acl_options_t* notify;
 	acl_options_t* provide_xfr;
+	acl_options_t* outgoing_interface;
+	uint8_t allow_axfr_fallback;
+	uint8_t notify_retry;
 };
 
 union acl_addr_storage {
@@ -98,6 +107,8 @@ struct acl_options {
 
 	/* options */
 	uint8_t use_axfr_only;
+	uint8_t allow_udp;
+	time_t ixfr_disabled;
 
 	/* ip address range */
 	const char* ip_address_spec;
@@ -147,6 +158,7 @@ struct config_parser_state {
 	acl_options_t* current_request_xfr;
 	acl_options_t* current_notify;
 	acl_options_t* current_provide_xfr;
+	acl_options_t* current_outgoing_interface;
 };
 
 extern config_parser_state_t* cfg_parser;
@@ -169,10 +181,10 @@ key_options_t* key_options_find(nsd_options_t* opt, const char* name);
 /* tsig must be inited, adds all keys in options to tsig. */
 void key_options_tsig_add(nsd_options_t* opt);
 
-/* check acl list, acl number that matches if passed(0..), 
+/* check acl list, acl number that matches if passed(0..),
  * or failure (-1) if dropped */
 /* the reason why (the acl) is returned too (or NULL) */
-int acl_check_incoming(acl_options_t* acl, struct query* q, 
+int acl_check_incoming(acl_options_t* acl, struct query* q,
 	acl_options_t** reason);
 int acl_addr_matches(acl_options_t* acl, struct query* q);
 int acl_key_matches(acl_options_t* acl, struct query* q);
@@ -197,5 +209,7 @@ int parse_acl_is_ipv6(const char* p);
 int parse_acl_range_type(char* ip, char** mask);
 /* parses subnet mask, fills 0 mask as well */
 void parse_acl_range_subnet(char* p, void* addr, int maxbits);
+/* clean up options */
+void nsd_options_destroy(nsd_options_t* opt);
 
 #endif /* OPTIONS_H */
