@@ -72,6 +72,37 @@ rdata_dname_to_string(buffer_type *output, rdata_atom_type rdata,
 	return 1;
 }
 
+static int
+rdata_dns_name_to_string(buffer_type *output, rdata_atom_type rdata,
+	rr_type* ATTR_UNUSED(rr))
+{
+	const uint8_t *data = rdata_atom_data(rdata);
+	size_t offset = 0;
+	uint8_t length = data[offset];
+	size_t i;
+
+	while (length > 0)
+	{
+		if (offset) /* concat label */
+			buffer_printf(output, ".");
+
+		for (i = 1; i <= length; ++i) {
+			char ch = (char) data[i+offset];
+			if (isprint((int)ch))
+				buffer_printf(output, "%c", ch);
+			else
+				buffer_printf(output, "\\%03u", (unsigned) ch);
+		}
+		/* next label */
+		offset = offset+length+1;
+		length = data[offset];
+	}
+
+	/* root label */
+	buffer_printf(output, ".");
+	return 1;
+}
+
 static int
 rdata_text_to_string(buffer_type *output, rdata_atom_type rdata,
 	rr_type* ATTR_UNUSED(rr))
@@ -83,7 +114,7 @@ rdata_text_to_string(buffer_type *output, rdata_atom_type rdata,
 	buffer_printf(output, "\"");
 	for (i = 1; i <= length; ++i) {
 		char ch = (char) data[i];
-		if (isprint(ch)) {
+		if (isprint((int)ch)) {
 			if (ch == '"' || ch == '\\') {
 				buffer_printf(output, "\\");
 			}
@@ -303,14 +334,14 @@ rdata_apl_to_string(buffer_type *output, rdata_atom_type rdata,
 
 	buffer_create_from(
 		&packet, rdata_atom_data(rdata), rdata_atom_size(rdata));
-	
+
 	if (buffer_available(&packet, 4)) {
 		uint16_t address_family = buffer_read_u16(&packet);
 		uint8_t prefix = buffer_read_u8(&packet);
 		uint8_t length = buffer_read_u8(&packet);
 		int negated = length & APL_NEGATION_MASK;
 		int af = -1;
-		
+
 		length &= APL_LENGTH_MASK;
 		switch (address_family) {
 		case 1: af = AF_INET; break;
@@ -343,13 +374,13 @@ rdata_services_to_string(buffer_type *output, rdata_atom_type rdata,
 
 	buffer_create_from(
 		&packet, rdata_atom_data(rdata), rdata_atom_size(rdata));
-	
+
 	if (buffer_available(&packet, 1)) {
 		uint8_t protocol_number = buffer_read_u8(&packet);
 		ssize_t bitmap_size = buffer_remaining(&packet);
 		uint8_t *bitmap = buffer_current(&packet);
 		struct protoent *proto = getprotobynumber(protocol_number);
-		
+
 		if (proto) {
 			int i;
 
@@ -389,7 +420,7 @@ rdata_ipsecgateway_to_string(buffer_type *output, rdata_atom_type rdata, rr_type
 	case IPSECKEY_DNAME:
 		rdata_dname_to_string(output, rdata, rr);
 		break;
-	default: 
+	default:
 		return 0;
 	}
 	return 1;
@@ -402,7 +433,7 @@ rdata_nxt_to_string(buffer_type *output, rdata_atom_type rdata,
 	size_t i;
 	uint8_t *bitmap = rdata_atom_data(rdata);
 	size_t bitmap_size = rdata_atom_size(rdata);
-	
+
 	for (i = 0; i < bitmap_size * 8; ++i) {
 		if (get_bit(bitmap, i)) {
 			buffer_printf(output, "%s ", rrtype_to_string(i));
@@ -421,7 +452,7 @@ rdata_nsec_to_string(buffer_type *output, rdata_atom_type rdata,
 	size_t saved_position = buffer_position(output);
 	buffer_type packet;
 	int insert_space = 0;
-	
+
 	buffer_create_from(
 		&packet, rdata_atom_data(rdata), rdata_atom_size(rdata));
 
@@ -430,7 +461,7 @@ rdata_nsec_to_string(buffer_type *output, rdata_atom_type rdata,
 		uint8_t bitmap_size = buffer_read_u8(&packet);
 		uint8_t *bitmap = buffer_current(&packet);
 		int i;
-		
+
 		if (!buffer_available(&packet, bitmap_size)) {
 			buffer_set_position(output, saved_position);
 			return 0;
@@ -476,6 +507,7 @@ rdata_unknown_to_string(buffer_type *output, rdata_atom_type rdata,
 
 static rdata_to_string_type rdata_to_string_table[RDATA_ZF_UNKNOWN + 1] = {
 	rdata_dname_to_string,
+	rdata_dns_name_to_string,
 	rdata_text_to_string,
 	rdata_byte_to_string,
 	rdata_short_to_string,
@@ -521,25 +553,31 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 	rdata_atom_type temp_rdatas[MAXRDATALEN];
 	rrtype_descriptor_type *descriptor = rrtype_descriptor_by_type(rrtype);
 	region_type *temp_region;
-	
+
 	assert(descriptor->maximum <= MAXRDATALEN);
 
 	if (!buffer_available(packet, data_size)) {
 		return -1;
 	}
-	
+
 	temp_region = region_create(xalloc, free);
-	
+
 	for (i = 0; i < descriptor->maximum; ++i) {
 		int is_domain = 0;
+		int is_normalized = 0;
 		int is_wirestore = 0;
 		size_t length = 0;
 		int required = i < descriptor->minimum;
-		
+
 		switch (rdata_atom_wireformat_type(rrtype, i)) {
 		case RDATA_WF_COMPRESSED_DNAME:
 		case RDATA_WF_UNCOMPRESSED_DNAME:
 			is_domain = 1;
+			is_normalized = 1;
+			break;
+		case RDATA_WF_LITERAL_DNAME:
+			is_domain = 1;
+			is_wirestore = 1;
 			break;
 		case RDATA_WF_BYTE:
 			length = sizeof(uint8_t);
@@ -592,6 +630,7 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 				break;
 			case IPSECKEY_DNAME:
 				is_domain = 1;
+				is_normalized = 1;
 				is_wirestore = 1;
 				break;
 			}
@@ -600,13 +639,13 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 
 		if (is_domain) {
 			const dname_type *dname;
-			
+
 			if (!required && buffer_position(packet) == end) {
 				break;
 			}
-			
+
 			dname = dname_make_from_packet(
-				temp_region, packet, 1, 1);
+				temp_region, packet, 1, is_normalized);
 			if (!dname || buffer_position(packet) > end) {
 				/* Error in domain name.  */
 				region_destroy(temp_region);
@@ -616,9 +655,9 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 				temp_rdatas[i].data = (uint16_t *) region_alloc(
                                 	region, sizeof(uint16_t) + dname->name_size);
 				temp_rdatas[i].data[0] = dname->name_size;
-				memcpy(temp_rdatas[i].data+1, dname_name(dname), 
+				memcpy(temp_rdatas[i].data+1, dname_name(dname),
 					dname->name_size);
-			} else 
+			} else
 				temp_rdatas[i].domain
 					= domain_table_insert(owners, dname);
 		} else {
@@ -631,7 +670,7 @@ rdata_wireformat_to_rdata_atoms(region_type *region,
 					break;
 				}
 			}
-			
+
 			temp_rdatas[i].data = (uint16_t *) region_alloc(
 				region, sizeof(uint16_t) + length);
 			temp_rdatas[i].data[0] = length;
@@ -685,7 +724,7 @@ rdata_atoms_to_unknown_string(buffer_type *output,
 			hex_to_string(
 				output, dname_name(dname), dname->name_size);
 		} else {
-			hex_to_string(output, rdata_atom_data(rdatas[i]), 
+			hex_to_string(output, rdata_atom_data(rdatas[i]),
 				rdata_atom_size(rdatas[i]));
 		}
 	}
