@@ -1114,8 +1114,8 @@ server_main(struct nsd *nsd)
 			/* only quit children after xfrd has acked */
 			send_children_quit(nsd);
 
+			namedb_fd_close(nsd->db);
 			region_destroy(server_region);
-			namedb_close(nsd->db);
 			server_shutdown(nsd);
 
 			/* ENOTREACH */
@@ -1165,7 +1165,7 @@ server_main(struct nsd *nsd)
 		close(xfrd_listener.fd);
 	}
 
-	namedb_close(nsd->db);
+	namedb_fd_close(nsd->db);
 	region_destroy(server_region);
 	server_shutdown(nsd);
 }
@@ -1324,7 +1324,7 @@ server_child(struct nsd *nsd)
 	bind8_stats(nsd);
 #endif /* BIND8_STATS */
 
-	namedb_close(nsd->db);
+	namedb_fd_close(nsd->db);
 	region_destroy(server_region);
 	server_shutdown(nsd);
 }
