@@ -14,11 +14,16 @@
 
 #include "region-allocator.h"
 
+#include <openssl/hmac.h>
+#include <openssl/sha.h>
+
 /*
  * Initialize OpenSSL support for TSIG.
  */
 int tsig_openssl_init(region_type *region);
 
+void tsig_openssl_finalize();
+
 #endif /* defined(TSIG) && defined(HAVE_SSL) */
 
 #endif /* _TSIG_H_ */
