@@ -249,16 +249,13 @@ zparser_conv_serial(region_type *region, const char *serialstr)
 {
 	uint16_t *r = NULL;
 	uint32_t serial;
-	long long v;
-	char *t;
+	const char *t;
 
-	v = strtoll(serialstr, &t, 10);
-	if (serialstr[0] == '\0' || *t != '\0' || v < 0 || v > UINT_MAX
-		|| (errno == ERANGE && (v == LLONG_MIN || v == LLONG_MAX)))
-	{
+	serial = strtoserial(serialstr, &t);
+	if (*t != '\0') {
 		zc_error_prev_line("serial is expected");
 	} else {
-		serial = htonl((uint32_t) v);
+		serial = htonl(serial);
 		r = alloc_rdata_init(region, &serial, sizeof(serial));
 	}
 	return r;
@@ -1126,34 +1123,33 @@ process_rr(void)
 		return 0;
 	}
 
+	/* Do we have the zone already? */
+	if (!zone)
+	{
+		zone = (zone_type *) region_alloc(parser->region,
+							  sizeof(zone_type));
+		zone->apex = parser->default_apex;
+		zone->soa_rrset = NULL;
+		zone->soa_nx_rrset = NULL;
+		zone->ns_rrset = NULL;
+		zone->opts = NULL;
+		zone->is_secure = 0;
+		zone->updated = 1;
+
+		zone->next = parser->db->zones;
+		parser->db->zones = zone;
+		parser->current_zone = zone;
+	}
+
 	if (rr->type == TYPE_SOA) {
 		/*
 		 * This is a SOA record, start a new zone or continue
 		 * an existing one.
 		 */
-		if (rr->owner->is_apex) {
-			/*
-			   should be error!
-			   every zone should be passed to zonec only once.
-			*/
-			zone = namedb_find_zone(parser->db, rr->owner);
-			assert(zone);
-		} else {
-			/* new zone part */
-			zone = (zone_type *) region_alloc(parser->region,
-							  sizeof(zone_type));
+		if (rr->owner->is_apex)
+			zc_error_prev_line("this SOA record was already encountered");
+		else if (rr->owner == parser->default_apex) {
 			zone->apex = rr->owner;
-			zone->soa_rrset = NULL;
-			zone->soa_nx_rrset = NULL;
-			zone->ns_rrset = NULL;
-			zone->opts = NULL;
-			zone->is_secure = 0;
-			zone->updated = 1;
-
-			/* insert in front of zone list */
-			zone->next = parser->db->zones;
-			parser->db->zones = zone;
-
 			rr->owner->is_apex = 1;
 		}
 
@@ -1230,18 +1226,17 @@ process_rr(void)
 #endif
 
 	/* Check we have SOA */
-	/* [XXX] this is dead code */
 	if (zone->soa_rrset == NULL) {
-		if (rr->type != TYPE_SOA) {
-			zc_error_prev_line(
-				"missing SOA record on top of the zone");
-		} else if (rr->owner != zone->apex) {
-			zc_error_prev_line(
-				"SOA record with invalid domain name");
-		} else {
-			zone->soa_rrset = rrset;
+		if (rr->type == TYPE_SOA) {
+			if (rr->owner != zone->apex) {
+				zc_error_prev_line(
+					"SOA record with invalid domain name");
+			} else {
+				zone->soa_rrset = rrset;
+			}
 		}
-	} else if (rr->type == TYPE_SOA) {
+	}
+	else if (rr->type == TYPE_SOA) {
 		zc_error_prev_line("duplicate SOA record discarded");
 		--rrset->rr_count;
 	}
@@ -1544,12 +1539,14 @@ main (int argc, char **argv)
 	check_dname(db);
 
 #ifndef NDEBUG
-	fprintf(stdout, "global_region: ");
-	region_dump_stats(global_region, stderr);
-	fprintf(stdout, "\n");
-	fprintf(stdout, "db->region: ");
-	region_dump_stats(db->region, stderr);
-	fprintf(stdout, "\n");
+	if (vflag > 0) {
+		fprintf(stdout, "global_region: ");
+		region_dump_stats(global_region, stdout);
+		fprintf(stdout, "\n");
+		fprintf(stdout, "db->region: ");
+		region_dump_stats(db->region, stdout);
+		fprintf(stdout, "\n");
+	}
 #endif /* NDEBUG */
 
 	/* Close the database */
